package dev.shadowsoffire.apotheosis.util;

import net.minecraft.world.entity.ai.attributes.AttributeInstance;
import net.minecraft.network.chat.Style;
import net.minecraft.util.Mth;
import dev.shadowsoffire.apotheosis.ench.table.ApothEnchantmentMenu;
import java.util.Set;
import net.minecraft.network.chat.MutableComponent;
import net.minecraft.world.item.enchantment.Enchantment;
import net.minecraft.world.level.LevelReader;
import dev.shadowsoffire.apotheosis.ench.api.IEnchantingBlock;
import dev.shadowsoffire.apotheosis.ench.table.EnchantingStatRegistry;
import net.minecraft.core.BlockPos;
import net.minecraft.world.level.block.state.BlockState;
import java.util.Iterator;
import java.util.stream.Stream;
import net.minecraft.world.entity.ai.attributes.AttributeMap;
import dev.shadowsoffire.placebo.reload.DynamicHolder;
import dev.shadowsoffire.attributeslib.api.IFormattableAttribute;
import dev.shadowsoffire.attributeslib.AttributesLib;
import net.minecraft.world.entity.ai.attributes.AttributeModifier;
import java.util.function.Predicate;
import com.google.common.base.Predicates;
import java.util.function.Function;
import java.util.Objects;
import net.minecraftforge.registries.ForgeRegistries;
import net.minecraft.network.chat.CommonComponents;
import net.minecraftforge.fml.loading.FMLEnvironment;
import net.minecraft.ChatFormatting;
import dev.shadowsoffire.apotheosis.adventure.loot.LootRarity;
import dev.shadowsoffire.apotheosis.adventure.loot.RarityRegistry;
import net.minecraft.network.chat.Component;
import java.util.function.Consumer;
import net.minecraft.world.entity.LivingEntity;
import net.minecraft.world.level.Level;

public class CommonTooltipUtil
{
    public static void appendBossData(final Level level, final LivingEntity entity, final Consumer<Component> tooltip) {
        final DynamicHolder<LootRarity> rarity = (DynamicHolder<LootRarity>)RarityRegistry.byLegacyId(entity.getPersistentData().m_128461_("apoth.rarity"));
        if (!rarity.isBound()) {
            return;
        }
        tooltip.accept((Component)Component.m_237110_("info.apotheosis.boss", new Object[] { ((LootRarity)rarity.get()).toComponent() }).m_130940_(ChatFormatting.GRAY));
        if (FMLEnvironment.production) {
            return;
        }
        tooltip.accept(CommonComponents.f_237098_);
        tooltip.accept((Component)Component.m_237115_("info.apotheosis.boss_modifiers").m_130940_(ChatFormatting.GRAY));
        final AttributeMap map = entity.m_21204_();
        final Stream stream = ForgeRegistries.ATTRIBUTES.getValues().stream();
        final AttributeMap obj = map;
        Objects.requireNonNull(obj);
        stream.map(obj::m_22146_).filter(Predicates.notNull()).forEach(inst -> {
            inst.m_22122_().iterator();
            final Iterator iterator;
            while (iterator.hasNext()) {
                final AttributeModifier modif = iterator.next();
                if (modif.m_22214_().startsWith("placebo_random_modifier_")) {
                    tooltip.accept(IFormattableAttribute.toComponent(inst.m_22099_(), modif, AttributesLib.getTooltipFlag()));
                }
            }
        });
    }
    
    public static void appendBlockStats(final Level world, final BlockState state, final BlockPos pos, final Consumer<Component> tooltip) {
        final float maxEterna = EnchantingStatRegistry.getMaxEterna(state, world, pos);
        final float eterna = EnchantingStatRegistry.getEterna(state, world, pos);
        final float quanta = EnchantingStatRegistry.getQuanta(state, world, pos);
        final float arcana = EnchantingStatRegistry.getArcana(state, world, pos);
        final float rectification = EnchantingStatRegistry.getQuantaRectification(state, world, pos);
        final int clues = EnchantingStatRegistry.getBonusClues(state, world, pos);
        final boolean treasure = ((IEnchantingBlock)state.m_60734_()).allowsTreasure(state, (LevelReader)world, pos);
        if (eterna != 0.0f || quanta != 0.0f || arcana != 0.0f || rectification != 0.0f || clues != 0) {
            tooltip.accept((Component)Component.m_237115_("info.apotheosis.ench_stats").m_130940_(ChatFormatting.GOLD));
        }
        if (eterna != 0.0f) {
            if (eterna > 0.0f) {
                tooltip.accept((Component)Component.m_237110_("info.apotheosis.eterna.p", new Object[] { String.format("%.2f", eterna), String.format("%.2f", maxEterna) }).m_130940_(ChatFormatting.GREEN));
            }
            else {
                tooltip.accept((Component)Component.m_237110_("info.apotheosis.eterna", new Object[] { String.format("%.2f", eterna) }).m_130940_(ChatFormatting.GREEN));
            }
        }
        if (quanta != 0.0f) {
            tooltip.accept((Component)Component.m_237110_("info.apotheosis.quanta" + ((quanta > 0.0f) ? ".p" : ""), new Object[] { String.format("%.2f", quanta) }).m_130940_(ChatFormatting.RED));
        }
        if (arcana != 0.0f) {
            tooltip.accept((Component)Component.m_237110_("info.apotheosis.arcana" + ((arcana > 0.0f) ? ".p" : ""), new Object[] { String.format("%.2f", arcana) }).m_130940_(ChatFormatting.DARK_PURPLE));
        }
        if (rectification != 0.0f) {
            tooltip.accept((Component)Component.m_237110_("info.apotheosis.rectification" + ((rectification > 0.0f) ? ".p" : ""), new Object[] { String.format("%.2f", rectification) }).m_130940_(ChatFormatting.YELLOW));
        }
        if (clues != 0) {
            tooltip.accept((Component)Component.m_237110_("info.apotheosis.clues" + ((clues > 0) ? ".p" : ""), new Object[] { String.format("%d", clues) }).m_130940_(ChatFormatting.DARK_AQUA));
        }
        if (treasure) {
            tooltip.accept((Component)Component.m_237115_("info.apotheosis.allows_treasure").m_130940_(ChatFormatting.GOLD));
        }
        final Set<Enchantment> blacklist = ((IEnchantingBlock)state.m_60734_()).getBlacklistedEnchantments(state, (LevelReader)world, pos);
        if (blacklist.size() > 0) {
            tooltip.accept((Component)Component.m_237115_("info.apotheosis.filter").m_130938_(s -> s.m_178520_(5812428)));
            for (final Enchantment e : blacklist) {
                final MutableComponent name = (MutableComponent)e.m_44700_(1);
                name.m_7360_().clear();
                name.m_130938_(s -> s.m_178520_(5798058));
                tooltip.accept((Component)Component.m_237113_(" - ").m_7220_((Component)name).m_130938_(s -> s.m_178520_(5798058)));
            }
        }
    }
    
    public static void appendTableStats(final Level world, final BlockPos pos, final Consumer<Component> tooltip) {
        final ApothEnchantmentMenu.TableStats stats = ApothEnchantmentMenu.gatherStats(world, pos, 0);
        tooltip.accept((Component)Component.m_237110_("info.apotheosis.eterna.t", new Object[] { String.format("%.2f", stats.eterna()), String.format("%.2f", EnchantingStatRegistry.getAbsoluteMaxEterna()) }).m_130940_(ChatFormatting.GREEN));
        tooltip.accept((Component)Component.m_237110_("info.apotheosis.quanta.t", new Object[] { String.format("%.2f", Math.min(100.0f, stats.quanta())) }).m_130940_(ChatFormatting.RED));
        tooltip.accept((Component)Component.m_237110_("info.apotheosis.arcana.t", new Object[] { String.format("%.2f", Math.min(100.0f, stats.arcana())) }).m_130940_(ChatFormatting.DARK_PURPLE));
        tooltip.accept((Component)Component.m_237110_("info.apotheosis.rectification.t", new Object[] { String.format("%.2f", Mth.m_14036_(stats.rectification(), -100.0f, 100.0f)) }).m_130940_(ChatFormatting.YELLOW));
        tooltip.accept((Component)Component.m_237110_("info.apotheosis.clues.t", new Object[] { String.format("%d", stats.clues()) }).m_130940_(ChatFormatting.DARK_AQUA));
    }
}
